# -*- coding: utf-8 -*-
"""
Created on Wed Jul 24 18:10:15 2019

@author: lealp
"""

import pandas as pd
import geopandas as gpd

try:
    from netcdf_gdf_setter import Base_class_space_time_netcdf_gdf
except:
    
    from .netcdf_gdf_setter import Base_class_space_time_netcdf_gdf
    
import numpy as np

import warnings
from shapely.geometry import Point



def make_fake_data(N=200):
    
    # creating example GeoDataframe for match-ups in EPSG 4326
    
    xx = np.random.randint(low=-60, high=-33, size=N)*1.105
    
    yy = np.random.randint(low=-4, high=20, size=N)*1.105
    
    df = pd.DataFrame({'lon':xx, 'lat':yy})
    
    
    df['geometry'] = df.apply(lambda x: Point(x['lon'], x['lat']), axis=1)
    
    
    gdf = gpd.GeoDataFrame(df, geometry='geometry', crs={'init':'epsg:4326'})
    
    gdf['Datetime'] = pd.date_range('2010-05-19', '2010-06-24',  periods=gdf.shape[0])
    
    
    return gdf



class Space_Time_Agg_over_polygons(Base_class_space_time_netcdf_gdf):
    
    def __init__(self, gdf, xarray_dataset=None, 
                 netcdf_temporal_coord_name='time',
                 geo_series_temporal_attribute_name = 'Datetime',
                 longitude_dimension='lon',
                 latitude_dimension='lat',
				):
        
        
        '''
        Class description:
        ------------------
        
            This class is a base class for ensuring that the given netcdf is in conformity with the algorithm.
            
            Ex: the Netcdf has to be sorted in ascending order for all dimensions (ex: time ,longitude, latitude). 
            Otherwise, the returned algorithm would return Nan values for all slices
            
            Also, it is mandatory for the user to define the longitude and latitude dimension Names (ex: 'lon', 'lat'),
            since there is no stadardization for defining these properties in the Netcdf files worldwide.
            
            
            
        Attributes:
            
            gdf (geodataframe):
            -----------------------
                The geodataframe object containing geometries to be analyzed.
            
            xarray_dataset (None): 
            -----------------------
            
                the Xarry Netcdf object to be analyzed
                
                
            netcdf_temporal_coord_name (str = 'time'): 
            -----------------------------------
            
                the name of the time dimension in the netcdf file
                
                
            geo_series_temporal_attribute_name(str = 'Datetime'):
            -----------------------------------
            
                the name of the time dimension in the geoseries file    
                
                
            longitude_dimension (str = 'lon'): 
            ----------------------------------
            
                the name of the longitude/horizontal dimension in the netcdf file
            
            
            latitude_dimension (str = 'lat'): 
            ----------------------------------
                the name of the latitude/vertical dimension in the netcdf file
        

        
        '''
        
        self.__netcdf_ds = xarray_dataset
        
        self._temporal_coords = netcdf_temporal_coord_name
        self.__geo_series_temporal_attribute_name = geo_series_temporal_attribute_name
        # setting Standard spatial_coords/dimension names 
        self.spatial_coords = dict(x = longitude_dimension,
                                   y = latitude_dimension)
        
        self.__gdf = gdf
        
        
        self.__netcdf_ds = self.netcdf_ds.sortby([self._temporal_coords, 
                                                  longitude_dimension,
                                                  latitude_dimension])
        
        self.netcdf_ds = self._slice_bounding_box()
    
    
    @ property
    
    def netcdf_ds(self):
        
        return self.__netcdf_ds
    
    @ netcdf_ds.setter
    
    def netcdf_ds(self, new_netcdf_ds):
        '''
        This property-setter alters the former netcdf_ds for the new gdf provided
        
        
        '''
     
        self.__netcdf_ds = new_netcdf_ds
    
    
    
    @ property
    
    def gdf(self):
        
        return self.__gdf
    
    @ gdf.setter
    
    def gdf(self, new_gdf):
        '''
        This property-setter alters the former GDF for the new gdf provided
        
        
        '''
     
        self.__gdf = new_gdf
    
    
    def _slice_bounding_box(self):
        
        xmin, ymin, xmax, ymax = self.gdf.geometry.total_bounds
        
        
        dx = float(self.coord_resolution(self.spatial_coords['x']))
        
        dy = float(self.coord_resolution(self.spatial_coords['y']))
        
        xmin -= dx # to ensure full pixel slicing
        
        xmax += dx # to ensure full pixel slicing 
        
        ymin -= dy # to ensure full pixel slicing 
        
        ymax += dy # to ensure full pixel slicing 
        
        
        result = self.netcdf_ds.sel({self.spatial_coords['x']:slice(xmin, xmax),
                                     self.spatial_coords['y']:slice(ymin, ymax)})
        
        return result
        
    
    def _slice_time_interval(self, time_init, final_time):
        
        
        result = self.netcdf_ds.sel({self._temporal_coords:slice(time_init, final_time)})
        

        return result
        
    
    
    
    def _make_time_space_aggregations(self, 
                                      geoDataFrame, 
                                      date_offset,
                                      netcdf_varnames, 
                                      agg_functions):
        
        Tmin = geoDataFrame[self.__geo_series_temporal_attribute_name].min()
        
        Tmax = geoDataFrame[self.__geo_series_temporal_attribute_name].max()
        
        time_init = Tmin - date_offset
                
        final_time = Tmax + date_offset
        
        netcdf_sliced = self._slice_time_interval(time_init, final_time)
        
        netcdf_sliced_as_gpd_geodataframe = self.netcdf_to_gdf(netcdf_sliced).copy()
            
            
        
        if not  netcdf_sliced_as_gpd_geodataframe.empty:
            
            
            
            
            sjoined = gpd.sjoin(netcdf_sliced_as_gpd_geodataframe, geoDataFrame, how="inner", op='within')
            
            if not sjoined.empty:
                
                
                sjoined_agg = sjoined.groupby([self.__geo_series_temporal_attribute_name, 'IDX'])[netcdf_varnames].agg(agg_functions)
                
                
                sjoined_agg['period_sliced'] = time_init.strftime("%Y/%m/%d %H:%M:%S") + ' <-> ' + final_time.strftime("%Y/%m/%d %H:%M:%S")
                
                
               
                
                return sjoined_agg
               
            else:
                pass
    
        else:
            pass

    
    
    def _evaluate_space_time_agg(self, 
                                 netcdf_varnames=['adg_443_qaa'], 
                                 dict_of_windows=dict(time_window='1D'),
                                 agg_functions=['nanmean','nansum','nanstd'],
                                 verbose=True):
        
        self.gdf2 = gpd.GeoDataFrame()
        
        
        date_offset = pd.tseries.frequencies.to_offset(dict_of_windows['time_window'])
        
        self.gdf['IDX'] = np.arange(self.gdf.shape[0])
        
        
        for idx, group in self.gdf.groupby(self.gdf[self.__geo_series_temporal_attribute_name].unique()):
            
            
            group2 = self._make_time_space_aggregations(group.copy(),
                                                        date_offset=date_offset,
                                                        netcdf_varnames=netcdf_varnames,
                                                        agg_functions=agg_functions)
            
            if verbose == True:
                print('\n'*3,  group2,  '\n'*2)
            
            self.gdf2 = self.gdf2.append(group2)
            
        with warnings.catch_warnings():
            warnings.filterwarnings('ignore')
            
            self.gdf2 = (self.gdf[['geometry', 'IDX', self.__geo_series_temporal_attribute_name]]
                          .merge(self.gdf2, 
                                 on=['IDX', self.__geo_series_temporal_attribute_name])
                        )
                          
                      
        self.gdf2.crs = self.gdf.crs
            
        
        return self.gdf2.copy()
    
    
    
def _base(gdf,
          netcdf,
          netcdf_varnames =['adg_443_qaa'],
          netcdf_temporal_coord_name='time',
          geo_series_temporal_attribute_name = 'Datetime',
          longitude_dimension='lon',
          latitude_dimension='lat',
          dict_of_windows=dict(time_window='M'),
          agg_functions=['mean', 'max', 'min', 'std'],
          verbose=True
                        
         ):
    
    
    
    
    Match_Upper = Space_Time_Agg_over_polygons(  gdf=gdf, 
                                                 xarray_dataset=netcdf, 
                                                 netcdf_temporal_coord_name='time',
                                                 geo_series_temporal_attribute_name = 'Datetime',
                                                 longitude_dimension='lon',
                                                 latitude_dimension='lat')
    
   
    return Match_Upper._evaluate_space_time_agg(netcdf_varnames=['adg_443_qaa'], 
                                                dict_of_windows=dict_of_windows,
                                                agg_functions=agg_functions,
                                                verbose=verbose)
            
            
            

def get_zonal_match_up(netcdf, 
					   gdf, 
                       netcdf_varnames =['adg_443_qaa'],
                       dict_of_windows=dict(time_window='5D'),
                       agg_functions=['mean', 'max', 'min', 'std'],
                       verbose=True
                       ):
	"""
    This function does Match - Up operations from centroids of Geoseries or GeoDataFrames over Netcdfs.
    
	Attributes:
	
		netcdf (xarray Dataset/Dataarray):
		--------------------------------------------------------------------------
		
		
		gdf (geopandas GeoDataFrame):
		--------------------------------------------------------------------------
		
		
		netcdf_varnames (list): a list containing the netcdf variable names to apply the aggregation.

			Example: netcdf_varnames=['adg_443_qaa']
		--------------------------------------------------------------------------
		
		
		dict_of_windows(dictionary)
		
		
			Example: dict_of_windows=dict(time_window='5D') # for 5 day window integration
			
				Other time integration options, follow pandas pattern (e.g.: 'Q', '3Y',...etc.)
							 
		--------------------------------------------------------------------------
		
		
		agg_functions(list):
		
			Example: agg_functions = ['mean', 'max', 'min', 'std']
			
		--------------------------------------------------------------------------
		
		
		verbose (bool): it sets the function to verbose (or not). 
		
			Example verbose=True
		--------------------------------------------------------------------------
		
		
		
	Returns:
		(geopandas GeoDataFrame)
		
	
    """
				   
	return _base(gdf=gdf, 
                netcdf=netcdf, 
                netcdf_varnames=netcdf_varnames,
                dict_of_windows=dict_of_windows,
                agg_functions=agg_functions,
                verbose=verbose
                )
                




# -*- coding: utf-8 -*-
"""
Created on Thu Sep 12 14:46:57 2019

@author: lealp
"""


import pandas as pd
import numpy as np

try:
    from netcdf_gdf_setter import Base_class_space_time_netcdf_gdf
except:
    
    from .netcdf_gdf_setter import Base_class_space_time_netcdf_gdf

import geopandas as gpd

    

class _Space_Time_Agg_over_centroids(Base_class_space_time_netcdf_gdf):
    
    def get_centroids_from_geometries_of_gdf(self, feature):
        '''
        Function description:
            This function returns the centroids (longitude and latitude) from the features of the geodataframe 
            
        returns (array, array):
            x, y
        
        '''
    
        
        return (feature.geometry.centroid.x, feature.geometry.centroid.y)
    
    def get_neighbors_from_axis(self, 
                                axis_mean_centroid, 
                                delta_coordinate, 
                                window_size):
        '''
        Parameters:
            
            axis_mean_centroid: the mean coordinate to generate the convolutional slicing window
            
            delta_coordinate: the delta to be applied in the given coordinate
        
        '''
        
        return (  axis_mean_centroid - (window_size*delta_coordinate), 
                  axis_mean_centroid + (window_size*delta_coordinate))
        
    def space_time_window(self, 
                          geo_series, 
                          dict_of_windows=dict(x_window=5, y_window=1, time_window=None, time_unit='D', time_init='1970-01-01'),
                          verbose=False):
        
        '''
        Function description:
            
            x_window: it is in data coordinate units. (i.e.: longitude units). Therefore for a x_window of size 1, 
                      the resultant reduction operation will be applied over the Point and a 'x -1' and 'x+1' longitude coordinates
            
            y_window: it is in data coordinate units. (i.e.: latitude units).  Therefore for a y_window of size 1, 
                      the resultant reduction operation will be applied over the Point and a 'y -1' and 'y+1' latitude coordinates
            
            time_unit (string): it sets the time offset to be evaluated in the Match-Up
            
            time_init: the Datetime string for temporal filtering.
            
            
            For standard application, we suggest the use of x_window=1 and y_window=1. This way, the Match-Up will be applied 
                    over a 3x3 matrix. 
        
        Returns: (result, x, y)
        
            result: the space-time filtered netcdf DataArray
            x, y: the longitude/latitude coordinates of the centroid used
        
        '''
        x, y  = self.get_centroids_from_geometries_of_gdf(geo_series)
        
        dy = self.coord_resolution(self.spatial_coords['y'])
    
        dx = self.coord_resolution(self.spatial_coords['x'])
        
        
        # getting geographic window
        
        lon_init, lon_final = self.get_neighbors_from_axis(x, dx, dict_of_windows['x_window'])
        
        lat_init, lat_final = self.get_neighbors_from_axis(y, dy, dict_of_windows['y_window'])
        
        # sort netcdf in ascending mode (especially for time axis) to ensure slicing operations.

        dims = self.get_dims_names_as_list()
        
        df_sorted = self.netcdf_ds.sortby(dims)
        
        
        # adding a condition to time. Whether or not to apply a time slicing.
        
        if dict_of_windows['time_window'] != None:
            
            
            time_mean = geo_series[self._datetime_parameters]
            
            time_delta = pd.Timedelta(dict_of_windows['time_window'], dict_of_windows['time_unit'])
            
            time_init = time_mean - time_delta
            
            final_time = time_mean + time_delta
            
            
            
            # slicing both time and space.
            
            
            result = df_sorted.sel( { self.spatial_coords['y']:slice(lat_init, lat_final), 
                                      self.spatial_coords['x']:slice(lon_init, lon_final),
                                      self.temporal_coords:slice(time_init, final_time)
                                    })
            
            
                
        else:
            
            # slice only spatially. Therefore, all time intervals will be aggregated.
            
            result = df_sorted.sel( { self.spatial_coords['y']:slice(lat_init, lat_final), 
                                      self.spatial_coords['x']:slice(lon_init, lon_final)
                                    })
            
        if verbose == True:
            print('\n\n', '\t'*2, 'Sliced time space window {0}'.format(dict(result.dims)), '\n'*2)
            
        return result.compute()
  
    def _aggregation_over_time_space(self, 
                                    geo_series,
                                    agg_functions=['nanmean','nansum','nanstd', 'nanmax', 'nanmin'], 
                                    netcdf_varnames=['air'],
                                    dict_of_windows=dict(x_window=5, 
                                                         y_window=1, 
                                                         time_window=None,        
                                                         time_unit='D'),
                                                         
                                    verbose=False):
                                        
        '''
        Function description:
            This is the main function for Match-Up (space-time aggregation) operations. It operates space and time 
            slicing operation over the netcdf dataset and applies a reduction operation over the nearest coordinates
            of each centroid of the geodataframe.
            
            
        Parameters:
            geo_series: 
            -----------
            
                the Geopandas geoseries object to be applied the operation
                
                
            agg_functions (list): 
            -----------
            
                The reduction functions that will be applied over the netcdf. 
                It must be a list of numpy name functions, or list containing 
                callables (user defined functions), or a mixture of both. 
                The order of the callables in the agg_function are not important.
                
                
            netcdf_varnames (list): 
            ----------------------
            
                the list of variables to apply the reduction over the netcdf
            
            dict_of_windows (dict): 
            ------------------------
            
                A dictionary containing the information for slicing the netcdf 
                according to longitude (x_window), latitude (y_window) and time 
                (time_window) operations.
                
               
                Dict attributes:
                ------------------------
                        
                    x_window: 
                    -----------------
                              the amount of x coordinates that will be used to create a bounding box around each centroid 
                              in the netcdf slicing (Attention: this operation is in 'X' coordinate units, and not in matrix units)
                              
                    y_window: 
                    -----------------
                            the amount of y coordinates that will be used to create a bounding box around each centroid 
                            in the netcdf slicing (Attention: this operation is in 'Y' coordinate units, and not in matrix units)
                    
                
                    time_window(float): 
                    -------------------
                    
                        the amount of time coordinates that will be used to create a bounding box 
                        around each centroid in the netcdf time slicing operation 
                        (Attention: this operation is in 'time_unit' coordinate units, 
                        and not in matrix units).
                        
                        For standard time_window=None. If None is applied, no time slice operation is applied. 
                        Only spatial reduction is applied. In this case, all time 
                        instants are aggregated for a given bounding box.
                
                
                time_unit(string): 
                -------------------
                
                    the string names that can be used in the pandas Timedelta operation
                                   
                                    {'ns': nanoseconds
                                    'us':microseconds
                                    'ms': miliseconds
                                    's': secibds
                                    'm': minutes
                                    'h':hours'
                                    'D': days
                                    'W': weeks
                                    'M': months
                                    'Y': years}
                                    
                                    
                verbose(Boolean): 
                -------------------
                
                    if False (standard), no slicing message operation is printed in the screen. 
                    If True, all slice operations are printed in the screen.
                                    
                                    
        Returns:
            A new geodataframe with the new columns. One column for each reduced operation for each netcdf_varname
        
        '''
                                        
        
        # checking the list of aggregation functions. 
        # If any is a custom function, a custom name will be created for each one separately and in order.
        
        agg_function_names = []                
        
        custom_counter=0
        for agg_function in agg_functions:
            if isinstance(agg_function, str):
                
                agg_function_names.append(agg_function)
            
            else:
                if custom_counter >0:
                    
                    agg_function_names.append('custom_agg_function'+'_' +str(custom_counter))
                
                else:                   
                    agg_function_names.append('custom_agg_function')
                    
                custom_counter +=1
        
        
        
        # Starting the aggregation algorithm:
        
            # gettin the sliced netcdf.
        
        df = self.space_time_window(geo_series, dict_of_windows, verbose=verbose)
        
        # creating a single multi Index for the pandas Series aggregated data:
        index = pd.MultiIndex.from_product( (netcdf_varnames ,agg_functions)) 
        
      
        
        # checking if the slice resulted in an empty Xarray-Dataset / DataArray
        
        if df.to_dataframe().empty == True:
            # if empty, return nan to the given feature
            
            agg_results = pd.Series(np.nan, index=index)
            
        
        else:

            # effective sliced pixels that are not masked.
            
            if df.to_dataframe().dropna().empty == True:
                if verbose == True:
                    
                    
                    print('all nan cases for the sliced netcdf','\n',
                          'Setting to NaN all aggregation functions')
                else:
                    pass
                
                agg_results = pd.Series(np.nan, index=index)
                 
            else:
                
                agg_results =  df[netcdf_varnames].to_dataframe().agg(agg_functions)
                
                agg_results = agg_results.melt() # converting to a pandas Series
    
                agg_results = pd.Series(agg_results['value'].values, index= index)
                
        # adding the aggregated data to the Geoseries (being NaNs or real aggregated values):
        
        geo_series = geo_series.append(agg_results)
        
        
        if verbose == True:
            print(geo_series, '\n'*2)
        
        
        return geo_series
    
    
    def __call__(self,
                 geo_series, 
                 netcdf_varnames=['adg_443_qaa'],
                 agg_functions=['nanmean','nansum','nanstd', 'nanmax', 'nanmin'], 
                 dict_of_windows=dict(x_window=5, y_window=1, time_window=None, time_unit='D'),
                 verbose=False):
        
        """
            Match up analyses can be done by directly calling the given class, as long as one passes
            
            the geo_series, and at least the netcdf_varname to be applied over.
            
            Parameters:
                netcdf_ds: the xarray netcdf dataframe object
                gdf: the geodataframe object
                agg_functions (list - optional): it can be a list of callables or a numpy callable
                netcdf_varnames: a list of all netcdf variables that must be evaluate by the match-up algorithm
                dict_of_windows(dict): a dictionary containing the windows to be applied for each dimension in the xarray
        
        """
        
        
        return self._aggregation_over_time_space(geo_series=geo_series, 
                                                 agg_functions=agg_functions, 
                                                 netcdf_varnames=netcdf_varnames, 
                                                 dict_of_windows=dict_of_windows, 
                                                 verbose=verbose)
        
        
        
        
class Match_up_over_centroids(object):
    def __init__(self, netcdf,
                 netcdf_temporal_coord_name='time',
                 geo_series_temporal_attribute_name = 'Datetime',
                 longitude_dimension='lon',
                 latitude_dimension='lat',):
        
        
        self.Match_Upper = _Space_Time_Agg_over_centroids(netcdf,
                                                         netcdf_temporal_coord_name=netcdf_temporal_coord_name,
                                                         geo_series_temporal_attribute_name=geo_series_temporal_attribute_name,
                                                         latitude_dimension=latitude_dimension,
                                                         longitude_dimension=longitude_dimension)
 

    def _base(self, geo_series,
                      netcdf_varnames =['adg_443_qaa'],
                      verbose=True,
                      dict_of_windows=dict(x_window=200, 
                                                  y_window=20000, 
                                                  time_window=50,                                                               
                                                  time_unit='d'),
                      agg_functions=['mean', 'max', 'min', 'std']
                            
                            ):
  
        
       
        return self.Match_Upper(geo_series=geo_series,
                           netcdf_varnames=netcdf_varnames,
                           verbose = verbose,
                           agg_functions=agg_functions,
                           dict_of_windows = dict_of_windows)
        
    
             
                         
                     
def get_match_up_over_centroids(netcdf, 
                                 gdf, 
                                 netcdf_varnames =['adg_443_qaa'],
                                 verbose=True,
                                 dict_of_windows=dict(x_window=200, 
                                                      y_window=20000,
                                                      time_window=50, 
                                                      time_unit='d'),
                                 agg_functions=['mean', 'max', 'min', 'std']):
                    
    """
    This function does Match - Up operations from centroids of Geoseries or GeoDataFrames over Netcdfs.
    
	Attributes:
	
		netcdf (xarray Dataset/Dataarray):
		--------------------------------------------------------------------------
		
		
		gdf (geopandas geoseries/GeoDataFrame):
		--------------------------------------------------------------------------
		
		
		netcdf_varnames (list): a list containing the netcdf variable names to apply the aggregation.

			Example: netcdf_varnames=['adg_443_qaa']
		--------------------------------------------------------------------------
		
		verbose (bool): it sets the function to verbose (or not). 
		
			Example verbose=True
		--------------------------------------------------------------------------
		
		
		dict_of_windows(dictionary)
		
		
			Example: dict_of_windows = dict(x_window=20, # this will look for a 'x' window of 20 coordinate units
                             y_window=20, # this will look for a 'y' window of 20 coordinate units
                             time_window=5,  # this will look for a 'time_unit' window of 5 coordinate units
                             time_unit='D')  # this sets the time unit for "Day". Other options use sintax from pandas.
							 
		--------------------------------------------------------------------------
		
		
		agg_functions(list):
		
			Example: agg_functions = ['mean', 'max', 'min', 'std']
			
		--------------------------------------------------------------------------
    
	Returns:
		(geopandas geoseries/GeoDataFrame)
		
	
    """
    
    Match_upper = Match_up_over_centroids(netcdf) 
    
    if isinstance(gdf, (gpd.geodataframe.GeoDataFrame)):
        
       return gdf.apply(lambda geo_series: Match_upper._base(geo_series = geo_series,
                                                      netcdf_varnames = netcdf_varnames,
                                                      verbose=verbose,
                                                      dict_of_windows=dict_of_windows, 
                                                      agg_functions=agg_functions),
    
                     axis=1  )
        
    else:
         return Match_upper._base(geo_series = gdf,
                           netcdf_varnames = netcdf_varnames,
                           verbose=verbose,
                           dict_of_windows=dict_of_windows)








# -*- coding: utf-8 -*-
"""
Created on Tue Aug 27 15:01:04 2019

@author: lealp
"""

import pandas as pd
import glob

from shapely.geometry import Point
import geopandas as gpd
import numpy as np
import os
import xarray as xr


from unittest import TestCase



from modules.match_ups_over_centroids import get_match_up_over_centroids
from modules.match_ups_over_polygons import get_zonal_match_up


def make_fake_data(N=200):
    
    # creating example GeoDataframe for match-ups in EPSG 4326
    
    xx = np.random.randint(low=-60, high=-33, size=N)*1.105
    
    yy = np.random.randint(low=-4, high=20, size=N)*1.105
    
    df = pd.DataFrame({'lon':xx, 'lat':yy})
    
    
    df['geometry'] = df.apply(lambda x: Point(x['lon'], x['lat']), axis=1)
    
    
    gdf = gpd.GeoDataFrame(df, geometry='geometry', crs={'init':'epsg:4326'})
    
    gdf['Datetime'] = pd.date_range('2010-05-19', '2010-06-24',  periods=gdf.shape[0])
    
    gdf.crs = {'init' :'epsg:4326'}
    
    
    return gdf


def get_netcdf_example():
    
    cpath = '/'.join([os.path.dirname(__file__), 'data'])
    path_file = glob.glob(cpath + '/*.nc' )
    
    
    return  xr.open_mfdataset(path_file[0])



class Tester_match_ups_over_centroids(TestCase):
    def test_match_ups_over_centroids(self):
        
        gdf = make_fake_data()
        
        
        CDOM = get_netcdf_example()
    
        
        gdf2 = get_match_up_over_centroids(netcdf=CDOM,
                                           gdf=gdf, 
                                           netcdf_varnames =['adg_443_qaa'],
                                           verbose=True,
                                           dict_of_windows=dict(x_window=50, 
                                                                  y_window=50, 
                                                                  time_window=10,                                                               
                                                                  time_unit='d'),
                                           agg_functions=['mean', 'max', 'min', 'std'])
        

    
        
        self.assertTrue(isinstance(gdf2, gpd.geodataframe.GeoDataFrame))    





class Tester_Space_Time_Agg_over_polygons(TestCase):
    def test_match_ups_over_polygons(self):
        
        gdf = make_fake_data(3)
        
        gdf.geometry = gdf.geometry.buffer(2.15) # in degrees
		
        CDOM = get_netcdf_example()
		
        
        gdf2 = get_zonal_match_up(gdf=gdf, 
                                  netcdf=CDOM,
                                  netcdf_varnames =['adg_443_qaa'],
                                  dict_of_windows=dict(time_window='1M'),
                                  agg_functions=['mean', 'max', 'min', 'std']
                                  
                                   )
    
        
        self.assertTrue(isinstance(gdf2, gpd.geodataframe.GeoDataFrame))    








#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages, Extension

import pandas as pd

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()
	

requirements = pd.read_csv('requirements_dev.txt').values.ravel().tolist()
	

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Philipe Riskalla Leal",
    author_email='leal.philipe@gmail.com',
    classifiers=[
        'Topic :: Education',                                        # this line follows the options given by: [1]
        "Topic :: Scientific/Engineering",    # this line follows the options given by: [1]		
        'Intended Audience :: Education',
		"Intended Audience :: Science/Research",
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',

        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="""This library operates Space-Time Match-Up operations over Netcdf-Xarray datasets and Geopandas-GeoDataFrames. \
				 It is a mandatoryr step for areas of study as geography, epidemiology, sociology, remote sensing, ecology, etc.""",
    
    install_requires=requirements,
    license="MIT license",
    
    include_package_data=True,
	
    keywords='time space Match Up xarray geopandas space-time reduction',
    name='time_space_reductions',
    packages=find_packages(include=['time_space_reductions']),
    setup_requires=setup_requirements,
    test_suite='nose.collector',
    tests_require=['nose'],
    url='https://github.com/PhilipeRLeal/time_space_reductions',
    version='1.3',
    zip_safe=False,
)




# [1]: https://pypi.org/classifiers/


# -*- coding: utf-8 -*-
"""
Created on Wed Jul 24 18:08:05 2019

@author: lealp
"""

import pandas as pd
pd.set_option('display.width', 50000)
pd.set_option('display.max_rows', 500)
pd.set_option('display.max_columns', 500)
from shapely.geometry import Point
import geopandas as gpd

import numpy as np



class Base_class_space_time_netcdf_gdf(object):
    def __init__(self, xarray_dataset=None, 
                 netcdf_temporal_coord_name='time',
                 geo_series_temporal_attribute_name = 'Datetime',
                 longitude_dimension='lon',
                 latitude_dimension='lat',
				):
        
        
        '''
        Class description:
        ------------------
        
            This class is a base class for ensuring that the given netcdf is in conformity with the algorithm.
            
            Ex: the Netcdf has to be sorted in ascending order for all dimensions (ex: time ,longitude, latitude). 
            Otherwise, the returned algorithm would return Nan values for all slices
            
            Also, it is mandatory for the user to define the longitude and latitude dimension Names (ex: 'lon', 'lat'),
            since there is no stadardization for defining these properties in the Netcdf files worldwide.
            
            
            
        Attributes:
            
            xarray_dataset (None): 
            -----------------------
            
                the Xarry Netcdf object to be analyzed
                
                
            netcdf_temporal_coord_name (str = 'time'): 
            -----------------------------------
            
                the name of the time dimension in the netcdf file
                
                
            geo_series_temporal_attribute_name(str = 'Datetime'):
            -----------------------------------
            
                the name of the time dimension in the geoseries file    
                
                
            longitude_dimension (str = 'lon'): 
            ----------------------------------
            
                the name of the longitude/horizontal dimension in the netcdf file
            
            
            latitude_dimension (str = 'lat'): 
            ----------------------------------
                the name of the latitude/vertical dimension in the netcdf file
        

        
        '''
        
        self.__netcdf_ds = xarray_dataset
        
        self._temporal_coords = netcdf_temporal_coord_name

        self._datetime_parameters = geo_series_temporal_attribute_name
        # setting Standard spatial_coords/dimension names 
        self.spatial_coords = dict(x = longitude_dimension,
                                   y = latitude_dimension)
    
    @ property
    
    def geo_series_temporal_attribute_name(self):
        return self._temporal_coords
    
    @geo_series_temporal_attribute_name.setter    
    
    def geo_series_temporal_attribute_name(self, new_geo_series_temporal_attribute_name):
        
        self._temporal_coords = new_geo_series_temporal_attribute_name
    
    
    
    @ property    
    def temporal_coords(self):
        return self._temporal_coords
    
    @ temporal_coords.setter
    def temporal_coords(self, new_temporal_coord_name):
        '''
        This property alters the time attribute name to be used in the Netcdf as a filter for Match up operations
        
        '''
        self._temporal_coords = new_temporal_coord_name
        
        
    def _to_datetime(self, string, format='%Y-%m-%d'):
		
        return pd.to_datetime(string, format=format)
    
	
    @ property    
    def netcdf_ds(self):
        return self.__netcdf_ds
    
    @ netcdf_ds.setter
    def netcdf_ds(self, new_netcdf_ds):
        
        self.__netcdf_ds = new_netcdf_ds
        
    
        
    def _convert_long_360_to_180(self, netcdf_ds, longitude_name):
        

        return netcdf_ds.assign_coords(longitude =( ((netcdf_ds[longitude_name] + 180) % 360) - 180))
    
    def _convert_lat_180_to_90(self, netcdf_ds, latitude_name):
        
    
        return netcdf_ds.assign_coords(latitude=( ((netcdf_ds[latitude_name] + 90) % 180) - 90))
        
    
    def get_dims_names_as_list(self):
        dims = np.array(self.netcdf_ds.dims).tolist()
        
        return dims


    
    def _get_coords_resolution(self, netcdf_ds, dimension):
        
        dx = np.diff(netcdf_ds[dimension][:2])[0]
            
        return np.abs(dx)

    def coord_resolution(self, dimension='longitude'):
        
        return self._get_coords_resolution(self.netcdf_ds, dimension)
    
    
    def netcdf_to_gdf(self, netcdf_ds):
        
        netcdf_as_dataframe = netcdf_ds.to_dataframe().reset_index()
        
        netcdf_as_dataframe.loc[:, 'geometry'] = None
        
        netcdf_as_dataframe['geometry'] = [Point([x,y]) for x,y in zip(netcdf_as_dataframe[self.spatial_coords['x']], netcdf_as_dataframe.reset_index()[self.spatial_coords['y']])]
        
        
        
        netcdf_as_dataframe = gpd.GeoDataFrame(netcdf_as_dataframe, crs=self.gdf.crs)
        
        return netcdf_as_dataframe
    
    